library AGS_delphi_tutorial;

{
  Fr eigene Plugins sind lediglich nderungen in U_Plugin erforderlich.
  U_AGS_Interface und auch diese Datei sind bereits an die folgenden
  Versionen angepasst:
  AGS    : v2.60.450 (2.6)
  Editor : v1
  Engine : v12

  Zustzliche Units knnen eingefgt werden, um bspw. Dialoge bereit zu
  stellen.

  Update bereits erstellter Plugins:
  U_Plugin durch eigene Units ersetzen.

  ---

  For implementing your own plugins please do only change U_Plugin.
  U_AGS_Interface and this file are compatible with these versions:
  AGS    : v2.60.450 (2.6)
  Editor : v1
  Engine : v12

  You can have additional units e.g. for dialogs.

  To update your plugin replace U_Plugin with your Units.
}


uses
  Windows,
  SysUtils,
  Classes,
  U_AGS_Interface in 'U_AGS_Interface.pas',
  U_Plugin in 'U_Plugin.pas',
  U_AGS_script_functions in 'U_AGS_script_functions.pas';

{$R *.RES}

// ============================================================================
// Editor - Export
// ============================================================================

// DLLEXPORT LPCSTR AGS_GetPluginName(void);
function AGS_GetPluginName: LPCSTR; cdecl;
begin
     Result := cPluginTitel;
end;

// DLLEXPORT int    AGS_EditorStartup (IAGSEditor *);
function AGS_EditorStartup (Editor: IAGSEditor): int; cdecl;
var
   s: string;
   ok: Boolean;
begin
     Result := -1;
     AGS_Editor := Editor;
     if not Assigned (AGS_Editor) then Exit;
     ok := AGS_Editor.version >= cMinEditorInterfaceVersion;
     if ok
     then try
             SetLength (s, 0);
             ok := EditorInitialization (s);
          except
             on E: Exception
             do begin
                s := Format ('%s: %s', [E.Classname, E.Message]);
                ok := False;
                end;
             end
     else s := Format ('AGS Editor Interface v%d needed, found: v%d',
                       [cMinEditorInterfaceVersion, AGS_Editor.version]);
     if not ok
     then begin
          MessageBox (AGS_Editor.GetWindowHandle, PChar (s), cPluginTitel,
                      MB_OK or MB_ICONERROR or MB_TASKMODAL);
          Exit;
          end;
     AGS_Editor.RegisterScriptHeader (cScriptHeader);
     Result := 0;
end;

// DLLEXPORT void   AGS_EditorShutdown (void);
procedure AGS_EditorShutdown; cdecl;
begin
     if not Assigned (AGS_Editor) then Exit;
     AGS_Editor.UnregisterScriptHeader (cScriptHeader);
     EditorFinalization;
     AGS_Editor := nil;
end;

// DLLEXPORT void   AGS_EditorProperties (HWND);
procedure AGS_EditorProperties (ParentWindow: HWND); cdecl;
begin
     EditorPluginProperties (ParentWindow);
end;

// DLLEXPORT int    AGS_EditorSaveGame (char *, int);
function AGS_EditorSaveGame (buffer: PChar; bufsize: int): int; cdecl;
begin
     Result := EditorSaveGame (buffer, bufsize);
end;

// DLLEXPORT void   AGS_EditorLoadGame (char *, int);
procedure AGS_EditorLoadGame (buffer: PChar; bufsize: int); cdecl;
begin
     EditorLoadGame (buffer, bufsize);
end;

// ============================================================================
// Engine - DLL-Export
// ============================================================================

// DLLEXPORT void   AGS_EngineStartup (IAGSEngine *);
procedure AGS_EngineStartup (Engine: IAGSEngine); cdecl;
var
   i: Integer;
   s: string;
   ok: Boolean;
begin
     AGS_Engine := Engine;
     if not Assigned (AGS_Engine) then Exit;
     ok := AGS_Engine.version >= cMinEngineInterfaceVersion;
     if ok
     then try
             SetLength (s, 0);
             ok := EngineInitialization (s);
          except
             on E: Exception
             do begin
                s := Format ('%s: %s', [E.Classname, E.Message]);
                ok := False;
                end;
             end
     else s := Format ('AGS Engine Interface v%d needed, found: v%d',
                       [cMinEngineInterfaceVersion, AGS_Engine.version]);
     if not ok
     then begin
          AGS_Engine.AbortGame (PChar (s));
          Exit;
          end;
     for i := Low (cScriptFunctions) to High (cScriptFunctions)
     do with cScriptFunctions [i]
        do AGS_Engine.RegisterScriptFunction (Name, Address);
end;

// DLLEXPORT int    AGS_EngineOnEvent (int, int);
function AGS_EngineOnEvent (event, data: int): int; cdecl;
begin
     Result := EngineOnEvent (event, data);
end;

// ============================================================================
// DLL - DLL-Export
// ============================================================================

function AGS_PluginV2: int;
begin
     Result := 1;
end;

exports
  // Editor
  AGS_GetPluginName,
  AGS_EditorStartup,
  AGS_EditorShutdown,
  AGS_EditorProperties,
  AGS_EditorSaveGame,
  AGS_EditorLoadGame,

  // Engine
  AGS_EngineStartup,
  AGS_EngineOnEvent,

  // DLL
  AGS_PluginV2
  ;

// DllMain
begin
end.


