unit U_AGS_Interface;

interface

uses Windows;

type
  int32  = Integer;
  int    = Integer;
  Pint32 = ^int32;

  LPDIRECTDRAW2 = Pointer;
  LPDIRECTDRAWSURFACE2 = Pointer;

  PBITMAP = ^char;

  PByte   = ^Byte;
  PPByte  = ^PByte; // unsigned char**

const
  // Mask types
  MASK_WALKABLE   = 1;
  MASK_WALKBEHIND = 2;
  MASK_HOTSPOT    = 3;
  // MASK_REGIONS is interface version 11 and above only
  MASK_REGIONS    = 4;


  // AGSCharacter.flags
  CHF_NOSCALING      = $01;
  CHF_FIXVIEW        = $02;     // between SetCharView and ReleaseCharView
  CHF_NOINTERACT     = $04;
  CHF_NODIAGONAL     = $08;
  CHF_ALWAYSIDLE     = $10;
  CHF_NOLIGHTING     = $20;
  CHF_NOTURNING      = $40;
  CHF_NOWALKBEHINDS  = $80;

  // AGSObject.flags
  OBJF_NOINTERACT    = 1;  // not clickable
  OBJF_NOWALKBEHINDS = 2;  // ignore walk-behinds

type
  TScriptFunctions = record
    Name: PChar;
    Address: Pointer;
  end;

// ============================================================================
// AGS - struct
// ============================================================================

type
  PAGSColor = ^TAGSColor;
  TAGSColor = record
    r, g, b, padding: Byte;
  end;

  PAGSGameOptions = ^TAGSGameOptions;
  TAGSGameOptions = record
    score: int32;      // player's current score
    usedmode: int32;   // set by ProcessClick to last cursor mode used
    disabled_user_interface: int32;  // >0 while in cutscene/etc
    gscript_timer: int32;    // obsolete
    debug_mode: int32;       // whether we're in debug mode
    globalvars: array [1..50] of int32;   // obsolete
    messagetime: int32;      // time left for auto-remove messages
    usedinv: int32;          // inventory item last used
    inv_top,inv_numdisp,inv_numorder,inv_numinline: int32;
    text_speed: int32;       // how quickly text is removed
    sierra_inv_color: int32; // background used to paint defualt inv window
    talkanim_speed: int32;   // animation speed of talking anims
    inv_item_wid,inv_item_hit: int32;  // set by SetInvDimensions
    speech_text_shadow: int32;         // colour of outline fonts (default black)
    swap_portrait_side: int32;         // sierra-style speech swap sides
    speech_textwindow_gui: int32;      // textwindow used for sierra-style speech
    follow_change_room_timer: int32;   // delay before moving following characters into new room
    totalscore: int32;           // maximum possible score
    skip_display: int32;         // how the user can skip normal Display windows
    no_multiloop_repeat: int32;  // for backwards compatibility
    roomscript_finished: int32;  // on_call finished in room
    used_inv_on: int32;          // inv item they clicked on
    no_textbg_when_voice: int32; // no textwindow bgrnd when voice speech is used
    max_dialogoption_width: int32; // max width of dialog options text window
    no_hicolor_fadein: int32;      // fade out but instant in for hi-color
    bgspeech_game_speed: int32;    // is background speech relative to game speed
    bgspeech_stay_on_display: int32; // whether to remove bg speech when DisplaySpeech is used
    unfactor_speech_from_textlength: int32; // remove "&10" when calculating time for text to stay
    mp3_loop_before_end: int32;    // loop this time before end of track (ms)
    speech_music_drop: int32;      // how much to drop music volume by when speech is played
    in_cutscene: int32;            // we are between a StartCutscene and EndCutscene
    fast_forward: int32;           // player has elected to skip cutscene
    room_width: int32;      // width of current room (320-res co-ordinates)
    room_height: int32;     // height of current room (320-res co-ordinates)
  end;

  PAGSCharacter = ^TAGSCharacter;
  TAGSCharacter = record
    defview: int32;
    talkview: int32;
    view: int32;
    room, prevroom: int32;
    x, y, wait: int32;
    flags: int32;
    following: short;
    followinfo: short;
    idleview: int32;           // the loop will be randomly picked
    idletime, idleleft: short; // num seconds idle before playing anim
    transparency: short;       // if character is transparent
    baseline: short;
    activeinv: int32;
    talkcolor: int32;
    reserved: array [1..10] of int32;
    loop, frame: short;
    walking, animating: short;
    walkspeed, animspeed: short;
    inv: array [1..301] of short;
    actx, acty: short;
    name: array [1..40] of char;
    scrname: array [1..20] of char;
    on: char;
  end;

  PAGSObject = ^TAGSObject;
  TAGSObject = record
    x,y: int32;
    transparent: int32;
    reserved: array [1..4] of int32;
    num: short;
    baseline: short;
    view,loop,frame: short;
    wait,moving: short;
    cycling: char;
    overall_speed: char;
    on: char;
    flags: char;
  end;

const
  // AGSViewFrame.flags
  FRAF_MIRRORED = 1;  // flipped left to right

type

  PAGSViewFrame = ^TAGSViewFrame;
  TAGSViewFrame = record
    pic: int32;            // sprite slot number
    xoffs, yoffs: short;
    speed: short;
    flags: int32;
    sound: int32;          // play sound when this frame comes round
    reserved_for_future: array [1..2] of int32;
  end;


// AGSMouseCursor.flags
const
  MCF_ANIMATEMOVE         = 1;
  MCF_DISABLED            = 2;
  MCF_STANDARD            = 4;
  MCF_ONLYANIMOVERHOTSPOT = 8;

type
  PAGSMouseCursor = ^TAGSMouseCursor;
  TAGSMouseCursor = record
    pic: int32;                       // sprite slot number
    hotx, hoty: short;                // x,y hotspot co-ordinates
    view: short;                      // view (for animating cursors) or -1
    name: array [0..9] of char;       // name of cursor mode
    flags: char;                      // MCF_flags above
  end;

// ============================================================================
// Editor - IAGSEditor
// ============================================================================

const
  cMinEditorInterfaceVersion = 1;

type

  IAGSEditor = class (TObject)
  public
    version: int32;
    pluginId: int32;   // used internally, do not touch this
    // get the HWND of the main editor frame
    function GetEditorHandle: HWND; virtual; stdcall;
    // get the HWND of the current active window
    function GetWindowHandle: HWND; virtual; stdcall;
    // add some script to the default header
    procedure RegisterScriptHeader (const header: PChar); virtual; stdcall;
    // de-register a script header (pass same pointer as when added)
    procedure UnregisterScriptHeader (const header: PChar); virtual; stdcall;
  end;

var AGS_Editor: IAGSEditor = nil;

// ============================================================================
// Engine - IAGSEngine
// ============================================================================

const
  // Below are interface 3 and later
  AGSE_KEYPRESS        = $0001;
  AGSE_MOUSECLICK      = $0002;
  AGSE_POSTSCREENDRAW  = $0004;
  // Below are interface 4 and later
  AGSE_PRESCREENDRAW   = $0008;
  // Below are interface 5 and later
  AGSE_SAVEGAME        = $0010;
  AGSE_RESTOREGAME     = $0020;
  // Below are interface 6 and later
  AGSE_PREGUIDRAW      = $0040;
  AGSE_LEAVEROOM       = $0080;
  AGSE_ENTERROOM       = $0100;
  AGSE_TRANSITIONIN    = $0200;
  AGSE_TRANSITIONOUT   = $0400;
  // Below are interface 12 and later
  AGSE_FINALSCREENDRAW = $0800;
  AGSE_TRANSLATETEXT   = $1000;
  AGSE_TOOHIGH         = $2000;

  // GetFontType font types
  FNT_INVALID = 0;
  FNT_SCI     = 1;
  FNT_TTF     = 2;

  // PlaySoundChannel sound types
  PSND_WAVE       = 1;
  PSND_MP3STREAM  = 2;
  PSND_MP3STATIC  = 3;
  PSND_OGGSTREAM  = 4;
  PSND_OGGSTATIC  = 5;
  PSND_MIDI       = 6;
  PSND_MOD        = 7;

const
  cMinEngineInterfaceVersion = 12;

type

  IAGSEngine = class (TObject)
  public
    version: int32;
    pluginId: int32;   // used internally, do not touch this
    // quit the game
    procedure AbortGame (const reason: PChar); virtual; stdcall;
    // get engine version
    function GetEngineVersion: PChar; virtual; stdcall;
    // register a script function with the system
    procedure RegisterScriptFunction (const name: PChar; address: Pointer); virtual; stdcall;
    // get game window handle
    function GetWindowHandle: HWND; virtual; stdcall;
    // get reference to main DirectDraw interface
    function GetDirectDraw2: LPDIRECTDRAW2; virtual; stdcall;
    // get the DDraw surface associated with a bitmap
    function GetBitmapSurface (bitmap: PBITMAP): LPDIRECTDRAWSURFACE2; virtual; stdcall;
    // get a reference to the screen bitmap
    function GetScreen: PBITMAP; virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 2 AND ABOVE ONLY
    // ask the engine to call back when a certain event happens
    procedure RequestEventHook (event: int32); virtual; stdcall;
    // get the options data saved in the editor
    function GetSavedData (buffer: PChar; bufsize: int32): int; virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 3 AND ABOVE ONLY
    // get the   functionscreen
    function GetVirtualScreen: PBITMAP; virtual; stdcall;
    // write text to the screen in the specified font and colour
    procedure DrawText (x, y, font, color: int32; text: PChar); virtual; stdcall;
    // get screen dimensions
    procedure GetScreenDimensions (var width, height, coldepth: int32); virtual; stdcall;
    // get screen surface to draw on
    function GetRawBitmapSurface (bitmap: PBITMAP): PPByte; virtual; stdcall;
    // release the surface
    procedure ReleaseBitmapSurface (bitmap: PBITMAP); virtual; stdcall;
    // get the current mouse co-ordinates
    procedure GetMousePosition (var x, y: int32); virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 4 AND ABOVE ONLY
    // get the current room number
    function GetCurrentRoom: int; virtual; stdcall;
    // get the number of background scenes in this room
    function GetNumBackgrounds: int; virtual; stdcall;
    // get the current background frame
    function GetCurrentBackground: int; virtual; stdcall;
    // get a background scene bitmap
    function GetBackgroundScene (Number: int32): PBITMAP; virtual; stdcall;
    // get dimensions of a bitmap
    procedure GetBitmapDimensions (bmp: PBITMAP; var width, height, coldepth: int32); virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 5 AND ABOVE ONLY
    // similar to fwrite - buffer, size, filehandle
    function FWrite (buffer: Pointer; Length, Handle: int32): int; virtual; stdcall;
    // similar to fread - buffer; size; filehandle
    function FRead (buffer: Pointer; Length, Handle: int32): int; virtual; stdcall;
    // print text; wrapping as usual
    procedure DrawTextWrapped (x, y, width, font, color: int32; const text: PChar); virtual; stdcall;
    // set the current active 'screen'
    procedure SetVirtualScreen (bitmap: PBITMAP); virtual; stdcall;
    // look up a word in the parser dictionary
    function LookupParserWord (const word: PChar): int; virtual; stdcall;
    // draw a bitmap to the active screen
    procedure BlitBitmap (x, y: int32; bitmap: PBITMAP; masked: int32); virtual; stdcall;
    // update the mouse and music
    procedure PollSystem; virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 6 AND ABOVE ONLY
    // get number of characters in game
    function GetNumCharacters: int; virtual; stdcall;
    // get reference to specified character struct
    function GetCharacter (charid: int32): PAGSCharacter; virtual; stdcall;
    // get reference to game struct
    function GetGameOptions: PAGSGameOptions; virtual; stdcall;
    // get reference to current palette
    function GetPalette: PAGSColor; virtual; stdcall;
    // update palette
    procedure SetPalette (start, finish: int32; palette: PAGSColor); virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 7 AND ABOVE ONLY
    // get the current player character
    function GetPlayerCharacter: int; virtual; stdcall;
    // adjust to viewport co-ordinates
    procedure RoomToViewport (var x, y: int32); virtual; stdcall;
    // adjust from viewport co-ordinates
    procedure ViewportToRoom (var x, y: int32); virtual; stdcall;
    // number of objects in current room
    function GetNumObjects: int; virtual; stdcall;
    // get reference to specified object
    function GetObject (number: int32): PAGSObject; virtual; stdcall;
    // get sprite graphic
    function GetSpriteGraphic (slot: int32): PBITMAP; virtual; stdcall;
    // create a new blank bitmap
    function CreateBlankBitmap (width, height, coldep: int32): PBITMAP; virtual; stdcall;
    // free a created bitamp
    procedure FreeBitmap (bmp: PBITMAP); virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 8 AND ABOVE ONLY
    // get one of the room area masks
    function GetRoomMask (which: int32): PBITMAP; virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 9 AND ABOVE ONLY
    // get a particular view frame
    function GetViewFrame (view, loop, frame: int32): PAGSViewFrame; virtual; stdcall;
    // get the walk-behind baseline of a specific WB area
    function GetWalkbehindBaseline(walkbehind: int32): int; virtual; stdcall;
    // get the address of a script function
    function GetScriptFunctionAddress(const funcName: PChar): Pointer; virtual; stdcall;
    // get the transparent colour of a bitmap
    function GetBitmapTransparentColor(BITMAP: PBITMAP): int; virtual; stdcall;
    // get the character scaling level at a particular point
    function GetAreaScaling (x, y: int32): int; virtual; stdcall;
    // equivalent to the text script function
    function IsGamePaused: int; virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 10 AND ABOVE ONLY
    // get the raw pixel value to use for the specified AGS colour
    function GetRawPixelColor (color: int32): int; virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 11 AND ABOVE ONLY
    // get the width / height of the specified sprite
    function GetSpriteWidth (slot: int32): int; virtual; stdcall;
    function GetSpriteHeight (slot: int32): int; virtual; stdcall;
    // get the dimensions of the specified string in the specified font
    procedure GetTextExtent (font: int32; const text: PChar; var width, height: int32); virtual; stdcall;
    // print a message to the debug console
    procedure PrintDebugConsole (const text: PChar); virtual; stdcall;
    // play a sound on the specified channel
    procedure PlaySoundChannel (channel, soundType, volume, loop: int32; const filename: PChar); virtual; stdcall;
    // same as text script function
    function IsChannelPlaying (channel: int32): int; virtual; stdcall;

    // *** BELOW ARE INTERFACE VERSION 12 AND ABOVE ONLY
    // invalidate a region of the virtual screen
    procedure MarkRegionDirty(left, top, right, bottom: int32); virtual; stdcall;
    // get mouse cursor details
    function GetMouseCursor(cursor: int32): PAGSMouseCursor; virtual; stdcall;
    // get the various components of a pixel
    procedure GetRawColorComponents(coldepth, color: int32; red, green, blue, alpha: Pint32); virtual; stdcall;
    // make a pixel colour from the supplied components
    function MakeRawColorPixel(coldepth, red, green, blue, alpha: int32): int; virtual; stdcall;
    // get whether the font is TTF or SCI
    function GetFontType(fontNum: int32): int; virtual; stdcall;
    // create a new dynamic sprite slot
    function CreateDynamicSprite(coldepth, width, height: int32): int; virtual; stdcall;
    // free a created dynamic sprite
    procedure DeleteDynamicSprite(slot: int32); virtual; stdcall;
    // check if a sprite has an alpha channel
    function IsSpriteAlphaBlended(slot: int32): int; virtual; stdcall;
  end;

var AGS_Engine: IAGSEngine = nil;

implementation

// ============================================================================
// Editor - IAGSEditor
// ============================================================================

function IAGSEditor.GetEditorHandle: HWND;
begin
     Result := 0;
end;

function IAGSEditor.GetWindowHandle: HWND;
begin
     Result := 0;
end;

procedure IAGSEditor.RegisterScriptHeader(const header: PChar);
begin

end;

procedure IAGSEditor.UnregisterScriptHeader(const header: PChar);
begin

end;

// ============================================================================
// Engine - IAGSEngine
// ============================================================================

procedure IAGSEngine.AbortGame(const reason: PChar);
begin

end;

procedure IAGSEngine.BlitBitmap(x, y: int32; bitmap: PBITMAP;
  masked: int32);
begin

end;

function IAGSEngine.CreateBlankBitmap(width, height, coldep: int32): PBITMAP;
begin
     Result := nil;
end;

function IAGSEngine.CreateDynamicSprite(coldepth, width,
  height: int32): int;
begin
     Result := 0;
end;

procedure IAGSEngine.DeleteDynamicSprite(slot: int32);
begin

end;

procedure IAGSEngine.DrawText(x, y, font, color: int32; text: PChar);
begin

end;

procedure IAGSEngine.DrawTextWrapped(x, y, width, font, color: int32;
  const text: PChar);
begin

end;

function IAGSEngine.FRead(buffer: Pointer; Length, Handle: int32): int;
begin
     Result := 0;
end;

procedure IAGSEngine.FreeBitmap(bmp: PBITMAP);
begin

end;

function IAGSEngine.FWrite(buffer: Pointer; Length, Handle: int32): int;
begin
     Result := 0;
end;

function IAGSEngine.GetAreaScaling(x, y: int32): int;
begin
     Result := 0;
end;

function IAGSEngine.GetBackgroundScene(Number: int32): PBITMAP;
begin
     Result := nil;
end;

procedure IAGSEngine.GetBitmapDimensions(bmp: PBITMAP; var width, height,
  coldepth: int32);
begin

end;

function IAGSEngine.GetBitmapSurface(bitmap: PBITMAP): LPDIRECTDRAWSURFACE2;
begin
     Result := nil;
end;

function IAGSEngine.GetBitmapTransparentColor(BITMAP: PBITMAP): int;
begin
     Result := 0;
end;

function IAGSEngine.GetCharacter(charid: int32): PAGSCharacter;
begin
     Result := nil;
end;

function IAGSEngine.GetCurrentBackground: int;
begin
     Result := 0;
end;

function IAGSEngine.GetCurrentRoom: int;
begin
     Result := 0;
end;

function IAGSEngine.GetDirectDraw2: LPDIRECTDRAW2;
begin
     Result := nil;
end;

function IAGSEngine.GetEngineVersion: PChar;
begin
     Result := '';
end;

function IAGSEngine.GetFontType(fontNum: int32): int;
begin
     Result := 0;
end;

function IAGSEngine.GetGameOptions: PAGSGameOptions;
begin
     Result := nil;
end;

function IAGSEngine.GetMouseCursor(cursor: int32): PAGSMouseCursor;
begin
     Result := nil;
end;

procedure IAGSEngine.GetMousePosition(var x, y: int32);
begin

end;

function IAGSEngine.GetNumBackgrounds: int;
begin
     Result := 0;
end;

function IAGSEngine.GetNumCharacters: int;
begin
     Result := 0;
end;

function IAGSEngine.GetNumObjects: int;
begin
     Result := 0;
end;

function IAGSEngine.GetObject(number: int32): PAGSObject;
begin
     Result := nil;
end;

function IAGSEngine.GetPalette: PAGSColor;
begin
     Result := nil;
end;

function IAGSEngine.GetPlayerCharacter: int;
begin
     Result := 0;
end;

function IAGSEngine.GetRawBitmapSurface(bitmap: PBITMAP): PPByte;
begin
     Result := nil;
end;

procedure IAGSEngine.GetRawColorComponents(coldepth, color: int32; red,
  green, blue, alpha: Pint32);
begin

end;

function IAGSEngine.GetRawPixelColor(color: int32): int;
begin
     Result := 0;
end;

function IAGSEngine.GetRoomMask(which: int32): PBITMAP;
begin
     Result := nil;
end;

function IAGSEngine.GetSavedData(buffer: PChar; bufsize: int32): int32;
begin
     Result := 0;
end;

function IAGSEngine.GetScreen: PBITMAP;
begin
     Result := nil;
end;

procedure IAGSEngine.GetScreenDimensions(var width, height, coldepth: int32);
begin

end;

function IAGSEngine.GetScriptFunctionAddress (const funcName: PChar): Pointer;
begin
     Result := nil;
end;

function IAGSEngine.GetSpriteGraphic(slot: int32): PBITMAP;
begin
     Result := nil;
end;

function IAGSEngine.GetSpriteHeight(slot: int32): int;
begin
     Result := 0;
end;

function IAGSEngine.GetSpriteWidth(slot: int32): int;
begin
     Result := 0;
end;

procedure IAGSEngine.GetTextExtent(font: int32; const text: PChar;
  var width, height: int32);
begin

end;

function IAGSEngine.GetViewFrame(view, loop, frame: int32): PAGSViewFrame;
begin
     Result := nil;
end;

function IAGSEngine.GetVirtualScreen: PBITMAP;
begin
     Result := nil;
end;

function IAGSEngine.GetWalkbehindBaseline(walkbehind: int32): int;
begin
     Result := 0;
end;

function IAGSEngine.GetWindowHandle: HWND;
begin
     Result := 0;
end;

function IAGSEngine.IsChannelPlaying(channel: int32): int;
begin
     Result := 0;
end;

function IAGSEngine.IsGamePaused: int;
begin
     Result := 0;
end;

function IAGSEngine.IsSpriteAlphaBlended(slot: int32): int;
begin
     Result := 0;
end;

function IAGSEngine.LookupParserWord(const word: PChar): int;
begin
     Result := 0;
end;

function IAGSEngine.MakeRawColorPixel(coldepth, red, green, blue,
  alpha: int32): int;
begin
     Result := 0;
end;

procedure IAGSEngine.MarkRegionDirty(left, top, right, bottom: int32);
begin

end;

procedure IAGSEngine.PlaySoundChannel(channel, soundType, volume,
  loop: int32; const filename: PChar);
begin

end;

procedure IAGSEngine.PollSystem;
begin

end;

procedure IAGSEngine.PrintDebugConsole(const text: PChar);
begin

end;

procedure IAGSEngine.RegisterScriptFunction(const name: PChar; address: Pointer);
begin

end;

procedure IAGSEngine.ReleaseBitmapSurface(bitmap: PBITMAP);
begin

end;

procedure IAGSEngine.RequestEventHook(event: int32);
begin

end;

procedure IAGSEngine.RoomToViewport(var x, y: int32);
begin

end;

procedure IAGSEngine.SetPalette(start, finish: int32; palette: PAGSColor);
begin

end;

procedure IAGSEngine.SetVirtualScreen(bitmap: PBITMAP);
begin

end;

procedure IAGSEngine.ViewportToRoom(var x, y: int32);
begin

end;

// ============================================================================
// End of unit
// ============================================================================

end.
