unit U_Plugin;

// if you want to use AGS script functions
// then remove // from the start of the following line: 
//{$DEFINE USE_AGS_SCRIPT_FUNCTIONS }

interface

uses Windows, U_AGS_Interface;

// Editor
function EditorInitialization (var Reason: string): Boolean;
procedure EditorPluginProperties (ParentWindow: HWND);
procedure EditorLoadGame (buffer: PChar; bufsize: int);
function EditorSaveGame (buffer: PChar; bufsize: int): int;
procedure EditorFinalization;

// Engine
function EngineInitialization (var Reason: string): Boolean;
function EngineOnEvent (event, data: int): int;

// ============================================================================
// Plugin
// ============================================================================

const // replace with your plugin name
  cPluginTitel = 'AGS Plugin Tutorial for Delphi Programmers (a-v-o)';

// --> declare your new script-functions
procedure SetSquareSize (SquareSize: int); cdecl;
function GetSquareSize: int; cdecl;

procedure SetTestText (Text: PChar); cdecl;
function GetTestText: PChar; cdecl;

const
  cScriptHeader    : PChar // --> Register functions for AGS editor header
                   = 'import void SetSquareSize (int SquareSize);'#13#10
                   + 'import int GetSquareSize ();'#13#10

                   + 'import void SetTestText (string Text);'#13#10
                   + 'import string GetTestText ();'#13#10
                   ;

  cScriptFunctions : array [1..4] of TScriptFunctions  // <-- set count of functions
                   = ( // --> Register functions for AGS engine
                      (Name: 'SetSquareSize'; Address: @SetSquareSize),
                      (Name: 'GetSquareSize'; Address: @GetSquareSize),

                      (Name: 'SetTestText'; Address: @SetTestText),
                      (Name: 'GetTestText'; Address: @GetTestText)
                     );

implementation

uses
  {$IFDEF USE_AGS_SCRIPT_FUNCTIONS}
  U_AGS_script_functions,
  {$ENDIF}
  SysUtils;

// ============================================================================
// Plugin-Methods
// ============================================================================

var SizeOfSquare : Integer = 50;  // used by DrawLightEffects

procedure SetSquareSize (SquareSize: int); cdecl;
begin
     SizeOfSquare := SquareSize;
end;

function GetSquareSize: int; cdecl;
begin
     Result := SizeOfSquare;
end;

// ---- string methods ---

var SavedString: string = '';

procedure SetTestText (Text: PChar); cdecl;
begin
     SavedString := Text;
end;

function GetTestText: PChar; cdecl;
begin
     Result := PChar (SavedString);
end;

// ============================================================================
// Draw on Bitmap
// ============================================================================

// Bitmap-structure:
// 8-bit-color-mode (256 colors):
//   each byte of the bitmap is an index into the palette
// (Hi-Color):     bits  15 14 13 12 11 10 9 8 7 6 5 4 3 2 1 0
// 15-bit-color-mode      -  R  R  R  R  R G G G G G B B B B B
// 16-bit-color-mode      R  R  R  R  R  G G G G G - B B B B B

type TColor5Bit = 0..31;

// function to convert RGB into AGS-15-bit-value
function RGB15 (Red, Green, Blue: TColor5Bit): Word;
begin
     // Red, Green, Blue: 0..31
     Result := (Red shl 10) + (Green shl 5) + Blue;
end;

// function to convert RGB into AGS-16-bit-value
function RGB16 (Red, Green, Blue: TColor5Bit): Word;
begin
     // Red, Green, Blue: 0..31
     Result := (Red shl 11) + (Green shl 6) + Blue;
end;

// function to convert AGS-15-bit-value into RGB
procedure AGS15toRGB (Color: Word; var Red, Green, Blue: TColor5Bit);
begin
     // Red, Green, Blue: 0..31
     Red := (Color shr 10) and 31;
     Green := (Color shr 5) and 31;
     Blue := Color and 31;
end;

// function to convert AGS-16-bit-value into RGB
procedure AGS16toRGB (Color: Word; var Red, Green, Blue: TColor5Bit);
begin
     // Red, Green, Blue: 0..31
     Red := (Color shr 11) and 31;
     Green := (Color shr 6) and 31;
     Blue := Color and 31;
end;

procedure DrawHorzLine (Row: PPByte; BitmapWidth, BitmapHeight, ColorDepth, xStart, xEnd, y: Integer);
var
   BytePixel: PByte;
   WordPixel: PWord;
begin
     if (0 <= y) and (y < BitmapHeight)                        // don't draw outside
     then begin
          Inc (Row, y);                                        // calculate row-pointer
          if xStart < 0 then xStart := 0;                      // don't draw outside
          if xEnd >= BitmapWidth then xEnd := BitmapWidth - 1; // don't draw outside
          if xStart > xEnd then Exit;                          // nothing to draw
          // depending on the color depth we need different drawing routines
          case ColorDepth
            of 8  : begin
                    BytePixel := Row^;                         // start of row
                    Inc (BytePixel, xStart);                   // calculate start of line
                    while xStart <= xEnd                       // draw line loop
                    do begin
                       BytePixel^ := 0;                        // use palette color 0 (black)
                       Inc (BytePixel);                        // next pixel
                       Inc (xStart);                           // next x-position
                       end;
                    end;
               15 : begin
                    WordPixel := PWord (Row^);                 // start of row
                    Inc (WordPixel, xStart);                   // calculate start of line
                    while xStart <= xEnd                       // draw line loop
                    do begin
                       WordPixel^ := RGB15 (31, 0, 0);         // calculate rgb-value (red)
                       Inc (WordPixel);                        // next pixel
                       Inc (xStart);                           // next x-position
                       end;
                    end;
               16 : begin
                    WordPixel := PWord (Row^);                 // start of row
                    Inc (WordPixel, xStart);                   // calculate start of line
                    while xStart <= xEnd                       // draw line loop
                    do begin
                       WordPixel^ := RGB16 (31, 0, 0);         // calculate rgb-value (red)
                       Inc (WordPixel);                        // next pixel
                       Inc (xStart);                           // next x-position
                       end;
                    end;
            end;
          end;
end;

procedure DrawVertLine (Row: PPByte; BitmapWidth, BitmapHeight, ColorDepth, x, yStart, yEnd: Integer);
var
   BytePixel: PByte;
   WordPixel: PWord;
begin
     if (0 <= x) and (x < BitmapWidth)                           // don't draw outside
     then begin
          if yStart < 0 then yStart := 0;                        // don't draw outside
          if yEnd >= BitmapHeight then yEnd := BitmapHeight - 1; // don't draw outside
          if yStart > yEnd then Exit;                            // nothing to draw
          Inc (Row, yStart);                                     // calculate row-pointer start of line
          while yStart <= yEnd                                   // draw line loop
          do begin
             // depending on the color depth we need different drawing routines
             case ColorDepth
               of 8  : begin
                       BytePixel := Row^;                        // start of row
                       Inc (BytePixel, x);                       // pixel of line
                       BytePixel^ := 0;                          // use palette color 0 (black)
                       end;
                  15 : begin
                       WordPixel := PWord (Row^);                // start of row
                       Inc (WordPixel, x);                       // pixel of line
                       WordPixel^ := RGB15 (31, 0, 0);           // calculate rgb-value (red)
                       end;
                  16 : begin
                       WordPixel := PWord (Row^);                // start of row
                       Inc (WordPixel, x);                       // pixel of line
                       WordPixel^ := RGB16 (31, 0, 0);           // calculate rgb-value (red)
                       end;
               end;
             Inc (Row);                                          // next row
             Inc (yStart);                                       // next y-position
             end;
          end;
end;

// example of a square (outlined) that can be moved with the mouse
procedure DrawEffects;
var
   VirtualScreen: PBITMAP;
   width, height, colordepth: int32;
   Bitmap: PPByte;
   mouseX, mouseY: int32;
   SquareLeft, SquareTop, SquareRight, SquareBottom: Integer;
begin
     // get screen dimensions and color mode
     AGS_Engine.GetScreenDimensions (width, height, colordepth);
     // get current mouse position
     AGS_Engine.GetMousePosition (mouseX, mouseY);
     // get a handle (?) to the virtual screen
     VirtualScreen := AGS_Engine.GetVirtualScreen;
     // lock virtual screen and get pointer to bitmap (must be released later)
     Bitmap := AGS_Engine.GetRawBitmapSurface (VirtualScreen);

     // calculate the position of the square
     // (here: mouse points to center of square)
     SquareLeft := mouseX - SizeOfSquare div 2;
     SquareTop  := mouseY - SizeOfSquare div 2;
     SquareRight := SquareLeft + SizeOfSquare;
     SquareBottom := SquareTop + SizeOfSquare;

     // draw top line of square
     DrawHorzLine (Bitmap, width, height, colordepth, SquareLeft, SquareRight, SquareTop);
     // draw bottom line of square
     DrawHorzLine (Bitmap, width, height, colordepth, SquareLeft, SquareRight, SquareBottom);
     // draw left line of square
     DrawVertLine (Bitmap, width, height, colordepth, SquareLeft, SquareTop, SquareBottom);
     // draw right line of square
     DrawVertLine (Bitmap, width, height, colordepth, SquareRight, SquareTop, SquareBottom);

     // unlock virtual screen
     AGS_Engine.ReleaseBitmapSurface (VirtualScreen);
end;

// ============================================================================
// Editor-Interface
// ============================================================================

function EditorInitialization (var Reason: string): Boolean;
begin
     // put here any code which is needed for initialization when running in the editor
     Result := True; // Initialization succesfull
end;

procedure EditorPluginProperties (ParentWindow: HWND);
begin
     // replace with your own properties
     MessageBox (ParentWindow, 'Just a tutorial', cPluginTitel, MB_OK or MB_ICONINFORMATION);
end;

procedure EditorLoadGame (buffer: PChar; bufsize: int);
begin
end;

function EditorSaveGame (buffer: PChar; bufsize: int): int;
begin
     Result := 0;
end;

procedure EditorFinalization;
begin
end;

// ============================================================================
// Engine-Interface
// ============================================================================

function EngineInitialization (var Reason: string): Boolean;
begin
     {$IFDEF USE_AGS_SCRIPT_FUNCTIONS}
     Result := InitScriptFunctions (Reason);
     if not Result then Exit;
     {$ENDIF}
     AGS_Engine.RequestEventHook (AGSE_PREGUIDRAW); // <-- see EngineOnEvent
     Result := True; // Initialization succesfull
end;

function EngineOnEvent (event, data: int): int;
begin
     Result := 0; // continue processing the event as normal
     // AGSE_PREGUIDRAW: draw after all scenery is drawn but before overlays and GUI is drawn
     if (event and AGSE_PREGUIDRAW) > 0 then DrawEffects;
end;

// ============================================================================
// End of Unit
// ============================================================================

end.
